//---------------------------------------------------------------------------
// TCommandsHandler.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include "Lib/Moochos/Moa/Files.h"
#include "CommandsHandler.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace TeenyContext { namespace CommandsHandler {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString CommandComFileName = "COMMAND.COM";
const AnsiString CmdExeFileName = "cmd.exe";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::Execute(const AnsiString& AVerb, const AnsiString& AFile, const AnsiString& AParameters, const AnsiString& ADirectory)
{
	SHELLEXECUTEINFO executeInfo = {0};
	executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	executeInfo.fMask = 0;
	if(lstrcmpi(AVerb.c_str(), "properties") == 0) {
		executeInfo.fMask |= SEE_MASK_INVOKEIDLIST;
	}
	executeInfo.hwnd = Application->Handle;
	executeInfo.lpVerb = static_cast<LPCTSTR>(AVerb.data());
	executeInfo.lpFile = static_cast<LPCTSTR>(AFile.data());
	executeInfo.lpParameters = static_cast<LPCTSTR>(AParameters.data());
	executeInfo.lpDirectory = static_cast<LPCTSTR>(ADirectory.data());
	executeInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&executeInfo)) {
		Abort();
	}
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::Copy(const AnsiString& AText)
{
	Clipboard()->AsText = AText;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommandsHandler::GetLocation(const AnsiString& AFileName)
{
	return ExtractFileDir(AFileName);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommandsHandler::GetNearestDir(const AnsiString& AFileName)
{
	if(DirectoryExists(AFileName)) {
		return AFileName;
	}
	return ExtractFileDir(AFileName);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommandsHandler::GetDefCommandPrompt()
{
	OSVERSIONINFO versionInfo = {0};
	versionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if(!GetVersionEx(&versionInfo)) {
		throw ECommandsHandlerError("The program cannot get the version information of the current operating system.");
	}
	switch(versionInfo.dwPlatformId) {
		case VER_PLATFORM_WIN32s:
		case VER_PLATFORM_WIN32_WINDOWS:
			return CommandComFileName;
		case VER_PLATFORM_WIN32_NT:
			return CmdExeFileName;
		default:
			throw ECommandsHandlerError("The program does not support the current operating system.");
	};
}
//---------------------------------------------------------------------------
__fastcall TCommandsHandler::TCommandsHandler() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TCommandsHandler::~TCommandsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::OpenLocation(const AnsiString& AFileName)
{
	Execute(AnsiString(), GetLocation(AFileName), AnsiString(), AnsiString());
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::ExploreLocation(const AnsiString& AFileName)
{
	Execute("explore", GetLocation(AFileName), AnsiString(), AnsiString());
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::CopyFullPath(const AnsiString& AFileName)
{
	Copy(AFileName);
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::CopyRoot(const AnsiString& AFileName)
{
	Copy(IncludeTrailingPathDelimiter(ExtractFileDrive(AFileName)));
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::CopyPath(const AnsiString& AFileName)
{
	Copy(ExtractFileDir(AFileName));
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::CopyName(const AnsiString& AFileName)
{
	Copy(ExtractFileName(AFileName));
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::StartDefCommandPrompt(const AnsiString& AFileName)
{
	Execute(AnsiString(), GetDefCommandPrompt(), AnsiString(), GetNearestDir(AFileName));
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::StartCommandCom(const AnsiString& AFileName)
{
	Execute(AnsiString(), CommandComFileName, AnsiString(), GetNearestDir(AFileName));
}
//---------------------------------------------------------------------------
void __fastcall TCommandsHandler::StartCmdExe(const AnsiString& AFileName)
{
	Execute(AnsiString(), CmdExeFileName, AnsiString(), GetNearestDir(AFileName));
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::TeenyContext::CommandsHandler
//---------------------------------------------------------------------------

