//---------------------------------------------------------------------------
// FileHandler.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Dialogs.h"
#include "Lib/Moochos/Moa/Files.h"
#include "FileHandler.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace TeenyContext { namespace FileHandler {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const int LargeFileCount = 6;
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Initialize()
{
	try {
		FFileDialog = new TOpenDialog(NULL);
		FileDialog->Title = "Choose File";
		FileDialog->Filter = "All|*";
		FileDialog->Options = FileDialog->Options << ofPathMustExist << ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Uninitialize()
{
	delete FFileDialog;
	FFileDialog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CreateAppInstance(const AnsiString& AFileName)
{
	Execute(AnsiString(), Application->ExeName, QuoteParameter(AFileName), GetCurrentDir());
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Execute(const AnsiString& AVerb, const AnsiString& AFile, const AnsiString& AParameters, const AnsiString& ADirectory)
{
	SHELLEXECUTEINFO executeInfo = {0};
	executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	executeInfo.fMask = 0;
	if(lstrcmpi(AVerb.c_str(), "properties") == 0) {
		executeInfo.fMask |= SEE_MASK_INVOKEIDLIST;
	}
	executeInfo.hwnd = Application->Handle;
	executeInfo.lpVerb = static_cast<LPCTSTR>(AVerb.data());
	executeInfo.lpFile = static_cast<LPCTSTR>(AFile.data());
	executeInfo.lpParameters = static_cast<LPCTSTR>(AParameters.data());
	executeInfo.lpDirectory = static_cast<LPCTSTR>(ADirectory.data());
	executeInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&executeInfo)) {
		Abort();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::QuoteParameter(const AnsiString& AParameter)
{
	if(AParameter.Length() == 0) {
		AParameter;
	}
	return AnsiString().sprintf("\"%s\"", AParameter.c_str());
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHandler::EnsureValidFileName(const AnsiString& AFileName)
{
	if(AFileName.Length() == 0) {
		return AFileName;
	}
	if(!IsAbsoluteFileName(AFileName)) {
		throw EFileHandlerError(AnsiString().sprintf(
			"The program cannot accept relative file names: %s.",
			AFileName.c_str()));
	}
	if(IsFileNameMask(AFileName)) {
		throw EFileHandlerError(AnsiString().sprintf(
			"The program cannot accept file name maeks: %s.",
			AFileName.c_str()));
	}
	AnsiString newFileName = ExtractLongPathName(ExpandFileName(AFileName));
	if(newFileName.Length() == 0) {
		throw EFileHandlerError(AnsiString().sprintf(
			"The program cannot get the long file name: %s.",
			AFileName.c_str()));
	}
	return newFileName;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::EnsureFileNameSpecified()
{
	if(FileName.Length() == 0) {
		throw EFileHandlerError("The file name is not specified.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetFileName(const AnsiString& AFileName)
{
	AnsiString newFileName = EnsureValidFileName(AFileName);
	if(newFileName == FileName) {
		return;
	}
	FFileName = newFileName;
	DoChange();
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::TFileHandler() :
	TObject(),
	FFileName(),
	FFileDialog(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::~TFileHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::New()
{
	CreateAppInstance(AnsiString());
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ChooseFile()
{
	FileDialog->FileName = AnsiString();
	if(!FileDialog->Execute()) {
		Abort();
	}
	SpecifyFiles(FileDialog->Files);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ChooseFolder()
{
	AnsiString folder = GetCurrentDir();
	if(!ChooseDirectory("Choose the target.", folder)) {
		Abort();
	}
	TPtr<TStrings> fileNames = new TStringList();
	fileNames->Add(folder);
	SpecifyFiles(fileNames);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SpecifyFiles(TStrings* AFileNames)
{
	if(AFileNames->Count == 0) {
		return;
	}
	int rest = AFileNames->Count;
	if(FileName.Length() == 0) {
		FileName = AFileNames->Strings[0];
		rest--;
	}
	if(rest >= LargeFileCount) {
		if(Application->MessageBox(
			AnsiString().sprintf(
				"The program creates %d instances of this application.\r\n\r\nContinue?",
				rest).c_str(),
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
	while(rest > 0) {
		CreateAppInstance(AFileNames->Strings[AFileNames->Count - rest]);
		rest--;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ShowProperties()
{
	EnsureFileNameSpecified();
	Execute("properties", FileName, AnsiString(), AnsiString());
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::TeenyContext::FileHandler
//---------------------------------------------------------------------------

